@@ -2,7 +2,7 @@
 //! ["Consensus on Transaction Commit"](https://www.microsoft.com/en-us/research/wp-content/uploads/2016/02/tr-2003-96.pdf)
 //! by Jim Gray and Leslie Lamport.
 
-use stateright::{Checker, Model, Property};
+use stateright::{Checker, Model, Property, Representative, Rewrite, RewritePlan};
 use std::collections::BTreeSet;
 use std::hash::Hash;
 use std::ops::Range;
@@ -23,7 +23,7 @@ struct TwoPhaseState {
 #[derive(Clone, Debug, Eq, Hash, Ord, PartialEq, PartialOrd)]
 enum Message { Prepared { rm: R }, Commit, Abort }
 
-#[derive(Clone, Debug, Eq, Hash, PartialEq)]
+#[derive(Clone, Debug, Eq, Hash, Ord, PartialEq, PartialOrd)]
 enum RmState { Working, Prepared, Committed, Aborted }
 
 #[derive(Clone, Debug, Eq, Hash, PartialEq)]
@@ -132,48 +132,78 @@ fn can_model_2pc() {
     let checker = TwoPhaseSys { rms: 0..5 }.checker().spawn_dfs().join();
     assert_eq!(checker.unique_state_count(), 8_832);
     checker.assert_properties();
+
+    // reverify the larger state space with symmetry reduction
+    let checker = TwoPhaseSys { rms: 0..5 }.checker().symmetry().spawn_dfs().join();
+    assert_eq!(checker.unique_state_count(), 665);
+    checker.assert_properties();
 }
 
-fn main() {
-    use clap::{App, Arg, SubCommand, value_t};
-
+fn main() -> Result<(), pico_args::Error> {
     env_logger::init_from_env(env_logger::Env::default()
         .default_filter_or("info")); // `RUST_LOG=${LEVEL}` env variable to override
 
-    let mut app = App::new("2pc")
-        .about("model check abstract two phase commit")
-        .subcommand(SubCommand::with_name("check")
-            .about("model check")
-            .arg(Arg::with_name("rm_count")
-                 .help("number of resource managers")
-                 .default_value("7")))
-        .subcommand(SubCommand::with_name("explore")
-            .about("interactively explore state space")
-            .arg(Arg::with_name("rm_count")
-                 .help("number of resource managers")
-                 .default_value("2"))
-            .arg(Arg::with_name("address")
-                .help("address Explorer service should listen upon")
-                .default_value("localhost:3000")));
-    let args = app.clone().get_matches();
-
-    match args.subcommand() {
-        ("check", Some(args)) => {
-            let rm_count = value_t!(args, "rm_count", usize).expect("rm_count");
+    let mut args = pico_args::Arguments::from_env();
+    match args.subcommand()?.as_deref() {
+        Some("check") => {
+            let rm_count = args.opt_free_from_str()?
+                .unwrap_or(2);
             println!("Checking two phase commit with {} resource managers.", rm_count);
             TwoPhaseSys { rms: 0..rm_count }.checker()
                 .threads(num_cpus::get()).spawn_dfs()
                 .report(&mut std::io::stdout());
         }
-        ("explore", Some(args)) => {
-            let rm_count = value_t!(args, "rm_count", usize).expect("rm_count");
-            let address = value_t!(args, "address", String).expect("address");
+        Some("check-sym") => {
+            let rm_count = args.opt_free_from_str()?
+                .unwrap_or(2);
+            println!("Checking two phase commit with {} resource managers using symmetry reduction.", rm_count);
+            TwoPhaseSys { rms: 0..rm_count }.checker()
+                .threads(num_cpus::get()).symmetry().spawn_dfs()
+                .report(&mut std::io::stdout());
+
+            // Implementing this trait enables symmetry reduction to speed up model checking (optional).
+            impl Representative for TwoPhaseState {
+                fn representative(&self) -> Self {
+                    let plan = RewritePlan::from_values_to_sort(&self.rm_state);
+                    Self {
+                        rm_state: plan.reindex(&self.rm_state),
+                        tm_state: self.tm_state.clone(),
+                        tm_prepared: plan.reindex(&self.tm_prepared),
+                        msgs: self.msgs.iter().map(|m| {
+                            match m {
+                                Message::Prepared { rm } =>
+                                    Message::Prepared { rm: plan.rewrite(rm) },
+                                Message::Commit => Message::Commit,
+                                Message::Abort => Message::Abort,
+                            }
+                        }).collect(),
+                    }
+                }
+            }
+            impl<T> Rewrite<T> for RmState {
+                fn rewrite<S>(&self, _: &RewritePlan<T,S>) -> Self {
+                    self.clone()
+                }
+            }
+        }
+        Some("explore") => {
+            let rm_count = args.opt_free_from_str()?
+                .unwrap_or(2);
+            let address = args.opt_free_from_str()?
+                .unwrap_or("localhost:3000".to_string());
             println!("Exploring state space for two phase commit with {} resource managers on {}.", rm_count, address);
             TwoPhaseSys { rms: 0..rm_count }.checker()
                 .threads(num_cpus::get())
                 .serve(address);
         }
-        _ => app.print_help().unwrap(),
+        _ => {
+            println!("USAGE:");
+            println!("  ./2pc check [RESOURCE_MANAGER_COUNT]");
+            println!("  ./2pc check-sym [RESOURCE_MANAGER_COUNT]");
+            println!("  ./2pc explore [RESOURCE_MANAGER_COUNT] [ADDRESS]");
+        }
     }
+
+    Ok(())
 }
 
