@@ -49,8 +49,8 @@
 use serde::{Deserialize, Serialize};
 use stateright::{Expectation, Model, Checker};
 use stateright::actor::{
-    Actor, ActorModel, DuplicatingNetwork, Id, model_peers, Out};
-use stateright::actor::register::{RegisterActor, RegisterActorState, RegisterMsg, RegisterMsg::*};
+    Actor, ActorModel, Id, majority, model_peers, Network,Out};
+use stateright::actor::register::{RegisterActor, RegisterMsg, RegisterMsg::*};
 use stateright::semantics::LinearizabilityTester;
 use stateright::semantics::register::Register;
 use stateright::util::{HashableHashMap, HashableHashSet};
@@ -138,7 +138,7 @@ impl Actor for PaxosActor {
                 // Simulate `Prepare` self-send.
                 state.ballot = (state.ballot.0 + 1, id);
                 // Simulate `Prepared` self-send.
-                state.prepares.insert(src, state.accepted);
+                state.prepares.insert(id, state.accepted);
 
                 o.broadcast(
                     &self.peer_ids,
@@ -155,7 +155,7 @@ impl Actor for PaxosActor {
             if ballot == state.ballot => {
                 let mut state = state.to_mut();
                 state.prepares.insert(src, last_accepted);
-                if state.prepares.len() > (self.peer_ids.len() + 1) / 2 {
+                if state.prepares.len() == majority(self.peer_ids.len() + 1) {
                     // This stage is best understood as "leadership handoff," in which this term's
                     // leader needs to ensure it does not contradict a decision (a quorum of
                     // accepts) from a previous term. Here's how:
@@ -181,7 +181,7 @@ impl Actor for PaxosActor {
                     // Simulate `Accept` self-send.
                     state.accepted = Some((ballot, proposal));
                     // Simulate `Accepted` self-send.
-                    state.accepts.insert(src);
+                    state.accepts.insert(id);
 
                     o.broadcast(&self.peer_ids, &Internal(Accept {
                         ballot,
@@ -198,7 +198,7 @@ impl Actor for PaxosActor {
             Internal(Accepted { ballot }) if ballot == state.ballot => {
                 let mut state = state.to_mut();
                 state.accepts.insert(src);
-                if state.accepts.len() > (self.peer_ids.len() + 1) / 2 {
+                if state.accepts.len() == majority(self.peer_ids.len() + 1) {
                     state.is_decided = true;
                     let proposal = state.proposal
                         .expect("proposal expected"); // See `Put` case above.
@@ -225,7 +225,7 @@ impl Actor for PaxosActor {
 struct PaxosModelCfg {
     client_count: usize,
     server_count: usize,
-    max_round: Round,
+    network: Network<<PaxosActor as Actor>::Msg>,
 }
 
 impl PaxosModelCfg {
@@ -248,29 +248,20 @@ impl PaxosModelCfg {
                         put_count: 1,
                         server_count: self.server_count,
                     }))
-            .duplicating_network(DuplicatingNetwork::No)
+            .init_network(self.network)
             .property(Expectation::Always, "linearizable", |_, state| {
                 state.history.serialized_history().is_some()
             })
             .property(Expectation::Sometimes, "value chosen", |_, state| {
-                for env in &state.network {
+                for env in state.network.iter_deliverable() {
                     if let RegisterMsg::GetOk(_req_id, value) = env.msg {
-                        if value != Value::default() { return true; }
+                        if *value != Value::default() { return true; }
                     }
                 }
                 false
             })
             .record_msg_in(RegisterMsg::record_returns)
             .record_msg_out(RegisterMsg::record_invocations)
-            .within_boundary(|cfg, state| {
-                state.actor_states.iter().all(|s| {
-                    if let RegisterActorState::Server(s) = &**s {
-                        s.ballot.0 <= cfg.max_round
-                    } else {
-                        true
-                    }
-                })
-            })
     }
 }
 
@@ -283,7 +274,7 @@ fn can_model_paxos() {
     let checker = PaxosModelCfg {
             client_count: 2,
             server_count: 3,
-            max_round: 3,
+            network: Network::new_unordered_nonduplicating([]),
         }
         .into_model().checker().spawn_bfs().join();
     checker.assert_properties();
@@ -297,13 +288,13 @@ fn can_model_paxos() {
         Deliver { src: 1.into(), dst: 2.into(), msg: Internal(Decided { ballot: (1, 1.into()), proposal: (4, 4.into(), 'B') }) },
         Deliver { src: 4.into(), dst: 2.into(), msg: Get(8) }
     ]);
-    assert_eq!(checker.unique_state_count(), 45_148);
+    assert_eq!(checker.unique_state_count(), 16_668);
 
     // DFS
     let checker = PaxosModelCfg {
             client_count: 2,
             server_count: 3,
-            max_round: 3,
+            network: Network::new_unordered_nonduplicating([]),
         }
         .into_model().checker().spawn_dfs().join();
     checker.assert_properties();
@@ -317,67 +308,54 @@ fn can_model_paxos() {
         Deliver { src: 1.into(), dst: 2.into(), msg: Internal(Decided { ballot: (1, 1.into()), proposal: (4, 4.into(), 'B') }) },
         Deliver { src: 4.into(), dst: 2.into(), msg: Get(8) }
     ]);
-    assert_eq!(checker.unique_state_count(), 45_148);
+    assert_eq!(checker.unique_state_count(), 16_668);
 }
 
-fn main() {
-    use clap::{App, AppSettings, Arg, SubCommand, value_t};
+fn main() -> Result<(), pico_args::Error> {
     use stateright::actor::spawn;
     use std::net::{SocketAddrV4, Ipv4Addr};
 
     env_logger::init_from_env(env_logger::Env::default()
         .default_filter_or("info")); // `RUST_LOG=${LEVEL}` env variable to override
 
-    let mut app = App::new("paxos")
-        .about("single decree paxos")
-        .setting(AppSettings::SubcommandRequiredElseHelp)
-        .subcommand(SubCommand::with_name("check")
-            .about("model check")
-            .arg(Arg::with_name("client_count")
-                .help("number of clients")
-                .default_value("2")))
-        .subcommand(SubCommand::with_name("explore")
-            .about("interactively explore state space")
-            .arg(Arg::with_name("client_count")
-                .help("number of clients")
-                .default_value("2"))
-            .arg(Arg::with_name("address")
-                .help("address Explorer service should listen upon")
-                .default_value("localhost:3000")))
-        .subcommand(SubCommand::with_name("spawn")
-            .about("spawn with messaging over UDP"));
-    let args = app.clone().get_matches();
-
-    match args.subcommand() {
-        ("check", Some(args)) => {
-            let client_count = std::cmp::min(
-                26, value_t!(args, "client_count", u8).expect("client count missing"));
+    let mut args = pico_args::Arguments::from_env();
+    match args.subcommand()?.as_deref() {
+        Some("check") => {
+            let client_count = args.opt_free_from_str()?
+                .unwrap_or(2);
+            let network = args.opt_free_from_str()?
+                .unwrap_or(Network::new_unordered_nonduplicating([]))
+                .into();
             println!("Model checking Single Decree Paxos with {} clients.",
                      client_count);
             PaxosModelCfg {
-                    client_count: client_count as usize,
+                    client_count,
                     server_count: 3,
-                    max_round: 3,
+                    network,
                 }
                 .into_model().checker().threads(num_cpus::get())
                 .spawn_dfs().report(&mut std::io::stdout());
         }
-        ("explore", Some(args)) => {
-            let client_count = std::cmp::min(
-                26, value_t!(args, "client_count", u8).expect("client count missing"));
-            let address = value_t!(args, "address", String).expect("address");
+        Some("explore") => {
+            let client_count = args.opt_free_from_str()?
+                .unwrap_or(2);
+            let address = args.opt_free_from_str()?
+                .unwrap_or("localhost:3000".to_string());
+            let network = args.opt_free_from_str()?
+                .unwrap_or(Network::new_unordered_nonduplicating([]))
+                .into();
             println!(
                 "Exploring state space for Single Decree Paxos with {} clients on {}.",
                 client_count, address);
             PaxosModelCfg {
-                    client_count: client_count as usize,
+                    client_count,
                     server_count: 3,
-                    max_round: 3,
+                    network,
                 }
                 .into_model().checker().threads(num_cpus::get())
                 .serve(address);
         }
-        ("spawn", Some(_args)) => {
+        Some("spawn") => {
             let port = 3000;
 
             println!("  A set of servers that implement Single Decree Paxos.");
@@ -402,6 +380,14 @@ fn main() {
                     (id2, PaxosActor { peer_ids: vec![id0, id1] }),
                 ]).unwrap();
         }
-        _ => app.print_help().unwrap(),
+        _ => {
+            println!("USAGE:");
+            println!("  ./paxos check [CLIENT_COUNT] [NETWORK]");
+            println!("  ./paxos explore [CLIENT_COUNT] [ADDRESS] [NETWORK]");
+            println!("  ./paxos spawn");
+            println!("NETWORK: {}", Network::<<PaxosActor as Actor>::Msg>::names().join(" | "));
+        }
     }
+
+    Ok(())
 }
