@@ -2,7 +2,7 @@
 //! provide consensus.
 
 use stateright::{Checker, Expectation, Model};
-use stateright::actor::{Actor, ActorModel, DuplicatingNetwork, Id, Out};
+use stateright::actor::{Actor, ActorModel, Id, Network, Out};
 use stateright::actor::register::{
     RegisterActor, RegisterMsg, RegisterMsg::*};
 use stateright::semantics::LinearizabilityTester;
@@ -41,6 +41,7 @@ impl Actor for SingleCopyActor {
 struct SingleCopyModelCfg {
     client_count: usize,
     server_count: usize,
+    network: Network<<SingleCopyActor as Actor>::Msg>,
 }
 
 impl SingleCopyModelCfg {
@@ -61,14 +62,14 @@ impl SingleCopyModelCfg {
                         put_count: 1,
                         server_count: self.server_count,
                     }))
-            .duplicating_network(DuplicatingNetwork::No)
+            .init_network(self.network)
             .property(Expectation::Always, "linearizable", |_, state| {
                 state.history.serialized_history().is_some()
             })
             .property(Expectation::Sometimes, "value chosen", |_, state| {
-                for env in &state.network {
+                for env in state.network.iter_deliverable() {
                     if let RegisterMsg::GetOk(_req_id, value) = env.msg {
-                        if value != Value::default() { return true; }
+                        if *value != Value::default() { return true; }
                     }
                 }
                 false
@@ -87,6 +88,7 @@ fn can_model_single_copy_register() {
     let checker = SingleCopyModelCfg {
             client_count: 2,
             server_count: 1,
+            network: Network::new_unordered_nonduplicating([]),
         }
         .into_model().checker().spawn_dfs().join();
     checker.assert_properties();
@@ -101,6 +103,7 @@ fn can_model_single_copy_register() {
     let checker = SingleCopyModelCfg {
             client_count: 2,
             server_count: 2,
+            network: Network::new_unordered_nonduplicating([]),
         }
         .into_model().checker().spawn_bfs().join();
     checker.assert_discovery("linearizable", vec![
@@ -118,62 +121,51 @@ fn can_model_single_copy_register() {
     assert_eq!(checker.unique_state_count(), 20);
 }
 
-fn main() {
-    use clap::{App, AppSettings, Arg, SubCommand, value_t};
+fn main() -> Result<(), pico_args::Error> {
     use stateright::actor::spawn;
     use std::net::{SocketAddrV4, Ipv4Addr};
 
     env_logger::init_from_env(env_logger::Env::default()
         .default_filter_or("info")); // `RUST_LOG=${LEVEL}` env variable to override
 
-    let mut app = App::new("wor")
-        .about("single-copy register")
-        .setting(AppSettings::SubcommandRequiredElseHelp)
-        .subcommand(SubCommand::with_name("check")
-            .about("model check")
-            .arg(Arg::with_name("client_count")
-                .help("number of gets")
-                .default_value("2")))
-        .subcommand(SubCommand::with_name("explore")
-            .about("interactively explore state space")
-            .arg(Arg::with_name("client_count")
-                .help("number of gets")
-                .default_value("2"))
-            .arg(Arg::with_name("address")
-                .help("address Explorer service should listen upon")
-                .default_value("localhost:3000")))
-        .subcommand(SubCommand::with_name("spawn")
-            .about("spawn with messaging over UDP"));
-    let args = app.clone().get_matches();
-
-    match args.subcommand() {
-        ("check", Some(args)) => {
-            let client_count = std::cmp::min(
-                26, value_t!(args, "client_count", u8).expect("client count missing"));
+    let mut args = pico_args::Arguments::from_env();
+    match args.subcommand()?.as_deref() {
+        Some("check") => {
+            let client_count = args.opt_free_from_str()?
+                .unwrap_or(2);
+            let network = args.opt_free_from_str()?
+                .unwrap_or(Network::new_unordered_nonduplicating([]))
+                .into();
             println!("Model checking a single-copy register with {} clients.",
                      client_count);
             SingleCopyModelCfg {
-                    client_count: client_count as usize,
+                    client_count,
                     server_count: 1,
+                    network,
                 }
                 .into_model().checker().threads(num_cpus::get())
                 .spawn_dfs().report(&mut std::io::stdout());
         }
-        ("explore", Some(args)) => {
-            let client_count = std::cmp::min(
-                26, value_t!(args, "client_count", u8).expect("client count missing"));
-            let address = value_t!(args, "address", String).expect("address");
+        Some("explore") => {
+            let client_count = args.opt_free_from_str()?
+                .unwrap_or(2);
+            let address = args.opt_free_from_str()?
+                .unwrap_or("localhost:3000".to_string());
+            let network = args.opt_free_from_str()?
+                .unwrap_or(Network::new_unordered_nonduplicating([]))
+                .into();
             println!(
                 "Exploring state space for single-copy register with {} clients on {}.",
                 client_count, address);
             SingleCopyModelCfg {
-                    client_count: client_count as usize,
+                    client_count,
                     server_count: 1,
+                    network,
                 }
                 .into_model().checker().threads(num_cpus::get())
                 .serve(address);
         }
-        ("spawn", Some(_args)) => {
+        Some("spawn") => {
             let port = 3000;
 
             println!("  A server that implements a single-copy register.");
@@ -192,6 +184,14 @@ fn main() {
                     (SocketAddrV4::new(Ipv4Addr::LOCALHOST, port), SingleCopyActor)
                 ]).unwrap();
         }
-        _ => app.print_help().unwrap(),
+        _ => {
+            println!("USAGE:");
+            println!("  ./single-copy-register check [CLIENT_COUNT]");
+            println!("  ./single-copy-register explore [CLIENT_COUNT] [ADDRESS] [NETWORK]");
+            println!("  ./single-copy-register spawn");
+            println!("NETWORK: {}", Network::<<SingleCopyActor as Actor>::Msg>::names().join(" | "));
+        }
     }
+
+    Ok(())
 }
