@@ -5,11 +5,12 @@ use crate::actor::{
     Actor,
     ActorModelState,
     Command,
+    Envelope,
     is_no_op,
     Id,
+    Network,
     Out,
 };
-use crate::util::HashableHashSet;
 use std::borrow::Cow;
 use std::fmt::{Debug, Display, Formatter};
 use std::hash::Hash;
@@ -22,15 +23,15 @@ use std::time::Duration;
 /// TLA](https://lamport.azurewebsites.net/tla/auxiliary/auxiliary.html) for a thorough
 /// introduction to that concept. Use `()` if history is not needed to define the relevant
 /// properties of this system.
+#[derive(Clone)]
 pub struct ActorModel<A, C = (), H = ()>
 where A: Actor,
       H: Clone + Debug + Hash,
 {
     pub actors: Vec<A>,
     pub cfg: C,
-    pub duplicating_network: DuplicatingNetwork,
     pub init_history: H,
-    pub init_network: Vec<Envelope<A::Msg>>,
+    pub init_network: Network<A::Msg>,
     pub lossy_network: LossyNetwork,
     pub properties: Vec<Property<ActorModel<A, C, H>>>,
     pub record_msg_in: fn(cfg: &C, history: &H, envelope: Envelope<&A::Msg>) -> Option<H>,
@@ -39,7 +40,7 @@ where A: Actor,
 }
 
 /// Indicates possible steps that an actor system can take as it evolves.
-#[derive(Clone, Debug, Eq, Hash, Ord, PartialEq, PartialOrd)]
+#[derive(Clone, Copy, Debug, Eq, Hash, Ord, PartialEq, PartialOrd)]
 pub enum ActorModelAction<Msg> {
     /// A message can be delivered to an actor.
     Deliver { src: Id, dst: Id, msg: Msg },
@@ -49,25 +50,12 @@ pub enum ActorModelAction<Msg> {
     Timeout(Id),
 }
 
-/// Indicates whether the network duplicates messages. If duplication is disabled, messages
-/// are forgotten once delivered, which can improve model checking performance.
-#[derive(Copy, Clone, PartialEq)]
-pub enum DuplicatingNetwork { Yes, No }
-
-/// Indicates the source and destination for a message.
-#[derive(Clone, Debug, Eq, Hash, Ord, PartialEq, PartialOrd)]
-#[derive(serde::Serialize)]
-pub struct Envelope<Msg> { pub src: Id, pub dst: Id, pub msg: Msg }
-
 /// Indicates whether the network loses messages. Note that as long as invariants do not check
 /// the network state, losing a message is indistinguishable from an unlimited delay, so in
 /// many cases you can improve model checking performance by not modeling message loss.
 #[derive(Copy, Clone, PartialEq)]
 pub enum LossyNetwork { Yes, No }
 
-/// Represents a network of messages.
-pub type Network<Msg> = HashableHashSet<Envelope<Msg>>;
-
 /// The specific timeout value is not relevant for model checking, so this helper can be used to
 /// generate an arbitrary timeout range. The specific value is subject to change, so this helper
 /// must only be used for model checking.
@@ -93,9 +81,8 @@ where A: Actor,
         ActorModel {
             actors: Vec::new(),
             cfg,
-            duplicating_network: DuplicatingNetwork::Yes,
             init_history,
-            init_network: Default::default(),
+            init_network: Network::new_unordered_duplicating([]),
             lossy_network: LossyNetwork::No,
             properties: Default::default(),
             record_msg_in: |_, _, _| None,
@@ -118,14 +105,8 @@ where A: Actor,
         self
     }
 
-    /// Defines whether the network duplicates messages or not.
-    pub fn duplicating_network(mut self, duplicating_network: DuplicatingNetwork) -> Self {
-        self.duplicating_network = duplicating_network;
-        self
-    }
-
     /// Defines the initial network.
-    pub fn init_network(mut self, init_network: Vec<Envelope<A::Msg>>) -> Self {
+    pub fn init_network(mut self, init_network: Network<A::Msg>) -> Self {
         self.init_network = init_network;
         self
     }
@@ -137,6 +118,7 @@ where A: Actor,
     }
 
     /// Adds a [`Property`] to this model.
+    #[allow(clippy::type_complexity)]
     pub fn property(mut self, expectation: Expectation, name: &'static str,
                      condition: fn(&ActorModel<A, C, H>, &ActorModelState<A, H>) -> bool) -> Self {
         self.properties.push(Property { expectation, name, condition });
@@ -185,7 +167,7 @@ where A: Actor,
                     {
                         state.history = history;
                     }
-                    state.network.insert(Envelope { src: id, dst, msg });
+                    state.network.send(Envelope { src: id, dst, msg });
                 },
                 Command::SetTimer(_) => {
                     // must use the index to infer how large as actor state may not be initialized yet
@@ -213,16 +195,10 @@ where A: Actor,
         let mut init_sys_state = ActorModelState {
             actor_states: Vec::with_capacity(self.actors.len()),
             history: self.init_history.clone(),
-            is_timer_set: Vec::new(),
-            network: Network::with_hasher(
-                crate::stable::build_hasher()), // for consistent discoveries
+            is_timer_set: vec![false; self.actors.len()],
+            network: self.init_network.clone(),
         };
 
-        // init the network
-        for e in self.init_network.clone() {
-            init_sys_state.network.insert(e);
-        }
-
         // init each actor
         for (index, actor) in self.actors.iter().enumerate() {
             let id = Id::from(index);
@@ -236,14 +212,20 @@ where A: Actor,
     }
 
     fn actions(&self, state: &Self::State, actions: &mut Vec<Self::Action>) {
-        for env in &state.network {
+        let mut prev_channel = None; // Only deliver the head of a channel.
+        for env in state.network.iter_deliverable() {
             // option 1: message is lost
             if self.lossy_network == LossyNetwork::Yes {
-                actions.push(ActorModelAction::Drop(env.clone()));
+                actions.push(ActorModelAction::Drop(env.to_cloned_msg()));
             }
 
             // option 2: message is delivered
-            if usize::from(env.dst) < self.actors.len() {
+            if usize::from(env.dst) < self.actors.len() { // ignored if recipient DNE
+                if matches!(self.init_network, Network::Ordered(_)) {
+                    let curr_channel = (env.src, env.dst);
+                    if prev_channel == Some(curr_channel) { continue; } // queued behind previous
+                    prev_channel = Some(curr_channel);
+                }
                 actions.push(ActorModelAction::Deliver { src: env.src, dst: env.dst, msg: env.msg.clone() });
             }
         }
@@ -260,7 +242,7 @@ where A: Actor,
         match action {
             ActorModelAction::Drop(env) => {
                 let mut next_state = last_sys_state.clone();
-                next_state.network.remove(&env);
+                next_state.network.on_drop(env);
                 Some(next_state)
             },
             ActorModelAction::Deliver { src, dst: id, msg } => {
@@ -286,15 +268,14 @@ where A: Actor,
                 // - Swap out revised actor state.
                 // - Track message input history.
                 // - Handle effect of commands on timers, network, and message output history.
+                //
+                // Strictly speaking, this state should be updated regardless of whether the
+                // actor and history updates are a no-op. The current implementation is only
+                // safe if invariants do not relate to the existence of envelopes on the
+                // network.
                 let mut next_sys_state = last_sys_state.clone();
-                if self.duplicating_network == DuplicatingNetwork::No {
-                    // Strictly speaking, this state should be updated regardless of whether the
-                    // actor and history updates are a no-op. The current implementation is only
-                    // safe if invariants do not relate to the existence of envelopes on the
-                    // network.
-                    let env = Envelope { src, dst: id, msg };
-                    next_sys_state.network.remove(&env);
-                }
+                let env = Envelope { src, dst: id, msg };
+                next_sys_state.network.on_deliver(env);
                 if let Cow::Owned(next_actor_state) = state {
                     next_sys_state.actor_states[index] = Arc::new(next_actor_state);
                 }
@@ -515,7 +496,7 @@ where A: Actor,
 #[cfg(test)]
 mod test {
     use super::*;
-    use crate::{Checker, StateRecorder};
+    use crate::{Checker, PathRecorder, StateRecorder};
     use crate::actor::actor_test_util::ping_pong::{PingPongCfg, PingPongMsg::*};
     use crate::actor::ActorModelAction::*;
     use std::collections::HashSet;
@@ -528,10 +509,11 @@ mod test {
 
         // helper to make the test more concise
         let states_and_network = |states: Vec<u32>, envelopes: Vec<Envelope<_>>| {
+            let is_timer_set = vec![false; states.len()];
             ActorModelState {
                 actor_states: states.into_iter().map(|s| Arc::new(s)).collect::<Vec<_>>(),
-                network: Network::from_iter(envelopes),
-                is_timer_set: Vec::new(),
+                network: Network::new_unordered_duplicating(envelopes),
+                is_timer_set,
                 history: (0_u32, 0_u32), // constant as `maintains_history: false`
             }
         };
@@ -654,7 +636,7 @@ mod test {
                 maintains_history: false,
             }
             .into_model()
-            .duplicating_network(DuplicatingNetwork::No)
+            .init_network(Network::new_unordered_nonduplicating([]))
             .lossy_network(LossyNetwork::No)
             .checker().spawn_bfs().join();
         assert_eq!(checker.unique_state_count(), 11);
@@ -687,7 +669,7 @@ mod test {
                 maintains_history: false,
             }
             .into_model()
-            .duplicating_network(DuplicatingNetwork::No)
+            .init_network(Network::new_unordered_nonduplicating([]))
             .lossy_network(LossyNetwork::No)
             .checker().spawn_bfs().join();
         assert_eq!(checker.unique_state_count(), 11);
@@ -704,12 +686,155 @@ mod test {
             ActorModel::new((), ())
                 .actor(())
                 .property(Expectation::Always, "unused", |_, _| true) // force full traversal
-                .init_network(vec![Envelope { src: 0.into(), dst: 99.into(), msg: () }])
+                .init_network(Network::new_unordered_duplicating([Envelope { src: 0.into(), dst: 99.into(), msg: () }]))
                 .checker().spawn_bfs().join()
                 .unique_state_count(),
             1);
     }
 
+    #[test]
+    fn handles_ordered_network_flag() {
+        #[derive(Clone)]
+        struct OrderedNetworkActor;
+        impl Actor for OrderedNetworkActor {
+            type Msg = u8;
+            type State = Vec<u8>;
+            fn on_start(&self, id: Id, o: &mut Out<Self>) -> Self::State {
+                if id == 0.into() {
+                    // Count down.
+                    o.send(1.into(), 2);
+                    o.send(1.into(), 1);
+                }
+                Vec::new()
+            }
+            fn on_msg(&self, _: Id, state: &mut Cow<Self::State>,
+                      _: Id, msg: Self::Msg, _: &mut Out<Self>)
+            {
+                state.to_mut().push(msg);
+            }
+        }
+
+        let model = ActorModel::new((), ())
+            .actors(vec![OrderedNetworkActor, OrderedNetworkActor])
+            .property(Expectation::Always, "", |_, _| true)
+            .init_network(Network::new_unordered_nonduplicating([]));
+
+        // Fewer states if network is ordered.
+        let (recorder, accessor) = StateRecorder::new_with_accessor();
+        model.clone()
+            .init_network(Network::new_ordered([]))
+            .checker().visitor(recorder).spawn_bfs().join();
+        let recipient_states: Vec<Vec<u8>> = accessor().into_iter().map(|s| {
+            (*s.actor_states[1]).clone()
+        }).collect();
+        assert_eq!(recipient_states, vec![
+            vec![],
+            vec![2],
+            vec![2, 1],
+        ]);
+
+        // More states if network is not ordered.
+        let (recorder, accessor) = StateRecorder::new_with_accessor();
+        model.clone()
+            .init_network(Network::new_unordered_nonduplicating([]))
+            .checker().visitor(recorder).spawn_bfs().join();
+        let recipient_states: Vec<Vec<u8>> = accessor().into_iter().map(|s| {
+            (*s.actor_states[1]).clone()
+        }).collect();
+        assert_eq!(recipient_states, vec![
+            vec![],
+            vec![2],
+            vec![1],
+            vec![2, 1],
+            vec![1, 2],
+        ]);
+    }
+
+    #[test]
+    fn unordered_network_has_a_bug() {
+        // Imagine that actor 1 sends two copies of a message to actor 2. An earlier implementation
+        // used a set to track envelopes in an unordered network even if it was lossy, and
+        // therefore could not distinguish between dropping/delivering 1 message vs multiple
+        // pending copies of the same message.
+
+        fn enumerate_action_sequences(lossy: LossyNetwork, init_network: Network<()>) -> HashSet<Vec<ActorModelAction<()>>> {
+            // There are two actors, and the first sends the same two messages to the second, which
+            // counts them.
+            struct A;
+            impl Actor for A {
+                type Msg = ();
+                type State = usize; // receipt count
+
+                fn on_start(&self, id: Id, o: &mut Out<Self>) -> Self::State {
+                    if id == 0.into() {
+                        o.send(1.into(), ());
+                        o.send(1.into(), ());
+                    }
+                    0
+                }
+                fn on_msg(&self, _: Id, state: &mut Cow<Self::State>, _: Id, _: Self::Msg, _: &mut Out<Self>) {
+                    *state.to_mut() += 1;
+                }
+            }
+
+            // Return the actions taken for each path based on the specified network
+            // characteristics.
+            let (recorder, accessor) = PathRecorder::new_with_accessor();
+            ActorModel::new((), ())
+                .actors([A, A])
+                .init_network(init_network)
+                .lossy_network(lossy)
+                .property(Expectation::Always, "force visiting all states", |_, _| true)
+                .within_boundary(|_, s| *s.actor_states[1] < 4)
+                .checker()
+                .visitor(recorder)
+                .spawn_dfs()
+                .join();
+            accessor()
+                .into_iter()
+                .map(|p| p.into_actions())
+                .collect()
+        }
+
+        // The actions are named here for brevity. These implement `Copy`.
+        let deliver = ActorModelAction::<()>::Deliver { src: 0.into(), msg: (), dst: 1.into() };
+        let drop = ActorModelAction::<()>::Drop(Envelope { src: 0.into(), msg: (), dst: 1.into() });
+
+        // Ordered networks can deliver/drop both messages.
+        let ordered_lossless = enumerate_action_sequences(LossyNetwork::No, Network::new_ordered([]));
+        assert!(ordered_lossless.contains(&vec![deliver, deliver]));
+        assert!(!ordered_lossless.contains(&vec![deliver, deliver, deliver]));
+        let ordered_lossy = enumerate_action_sequences(LossyNetwork::Yes, Network::new_ordered([]));
+        assert!(ordered_lossy.contains(&vec![deliver, deliver]));
+        assert!(ordered_lossy.contains(&vec![deliver, drop])); // same state as "drop, deliver"
+        assert!(ordered_lossy.contains(&vec![drop, drop]));
+
+        // Unordered duplicating networks can deliver/drop duplicates.
+        //
+        // IMPORTANT: in the context of a duplicating network, "dropping" must either entail:
+        //            (1) a no-op or (2) never deliving again. This implementation favors the
+        //            latter.
+        let unord_dup_lossless = enumerate_action_sequences(
+            LossyNetwork::No, Network::new_unordered_duplicating([]));
+        assert!(unord_dup_lossless.contains(&vec![deliver, deliver, deliver]));
+        let unord_dup_lossy = enumerate_action_sequences(
+            LossyNetwork::Yes, Network::new_unordered_duplicating([]));
+        assert!(unord_dup_lossy.contains(&vec![deliver, deliver, deliver]));
+        assert!(unord_dup_lossy.contains(&vec![deliver, deliver, drop]));
+        assert!(unord_dup_lossy.contains(&vec![deliver, drop]));
+        assert!(unord_dup_lossy.contains(&vec![drop]));
+        assert!(!unord_dup_lossy.contains(&vec![drop, deliver])); // b/c drop means "never deliver again"
+
+        // Unordered nonduplicating networks can deliver/drop both messages.
+        let unord_nondup_lossless = enumerate_action_sequences(
+            LossyNetwork::No, Network::new_unordered_nonduplicating([]));
+        assert!(unord_nondup_lossless.contains(&vec![deliver, deliver]));
+        let unord_nondup_lossy = enumerate_action_sequences(
+            LossyNetwork::Yes, Network::new_unordered_nonduplicating([]));
+        assert!(unord_nondup_lossy.contains(&vec![deliver, drop]));
+        assert!(unord_nondup_lossy.contains(&vec![drop, drop]));
+    }
+
     #[test]
     fn resets_timer() {
         struct TestActor;
@@ -738,7 +863,6 @@ mod test {
 mod choice_test {
     use choice::Choice;
     use crate::{Checker, Model, StateRecorder};
-    use crate::actor::DuplicatingNetwork;
     use super::*;
 
     #[derive(Clone)]
@@ -794,7 +918,7 @@ mod choice_test {
             .actor(Choice::new(A { b: Id::from(1) }))
             .actor(Choice::new(B { c: Id::from(2) }).or())
             .actor(Choice::new(C { a: Id::from(0) }).or().or())
-            .duplicating_network(DuplicatingNetwork::No)
+            .init_network(Network::new_unordered_nonduplicating([]))
             .record_msg_out(|_, out_count, _| Some(out_count + 1))
             .property(Expectation::Always, "true", |_, _| true)
             .within_boundary(|_, state| state.history < 8);
