@@ -1,18 +1,25 @@
 //! Private module for selective re-export.
 
 mod bfs;
-use crate::{Expectation, Model};
 mod dfs;
 mod explorer;
 mod path;
+mod representative;
+mod rewrite;
+mod rewrite_plan;
 mod visitor;
+
+use crate::{Expectation, Model};
 use std::collections::HashMap;
 use std::fmt::Debug;
 use std::hash::Hash;
 use std::num::NonZeroUsize;
 use std::time::Instant;
 
+pub use rewrite::*;
 pub use path::*;
+pub use representative::*;
+pub use rewrite_plan::*;
 pub use visitor::*;
 
 /// A [`Model`] [`Checker`] builder. Instantiable via the [`Model::checker`] method.
@@ -27,6 +34,8 @@ pub use visitor::*;
               Consider calling spawn_bfs() or spawn_dfs()."]
 pub struct CheckerBuilder<M: Model> {
     model: M,
+    #[allow(clippy::type_complexity)]
+    symmetry: Option<fn(&M::State) -> M::State>,
     target_state_count: Option<NonZeroUsize>,
     thread_count: usize,
     visitor: Option<Box<dyn CheckerVisitor<M> + Send + Sync>>,
@@ -36,6 +45,7 @@ impl<M: Model> CheckerBuilder<M> {
         Self {
             model,
             target_state_count: None,
+            symmetry: None,
             thread_count: 1,
             visitor: None,
         }
@@ -134,6 +144,22 @@ impl<M: Model> CheckerBuilder<M> {
         dfs::DfsChecker::spawn(self)
     }
 
+    /// Enables symmetry reduction. Requires the [model state] to implement [`Representative`].
+    ///
+    /// [model state]: crate::Model::State
+    pub fn symmetry(self) -> Self
+    where M::State: Representative,
+    {
+        self.symmetry_fn(Representative::representative)
+    }
+
+    /// Enables symmetry reduction based on a representative function.
+    ///
+    /// [model state]: crate::Model::State
+    pub fn symmetry_fn(self, representative: fn(&M::State) -> M::State) -> Self {
+        Self { symmetry: Some(representative), .. self }
+    }
+
     /// Sets the number of states that the checker should aim to generate. For performance reasons
     /// the checker may exceed this number, but it will never generate fewer states if more exist.
     pub fn target_state_count(self, count: usize) -> Self {
@@ -285,7 +311,7 @@ pub trait Checker<M: Model> {
                         });
                         let is_path_terminal = {
                             let mut actions = Vec::new();
-                            self.model().actions(&states.last().unwrap(), &mut actions);
+                            self.model().actions(states.last().unwrap(), &mut actions);
                             actions.is_empty()
                         };
                         if !is_liveness_satisfied && is_path_terminal { return }
