@@ -34,18 +34,25 @@ where M: Model + Send + Sync + 'static,
 {
     pub(crate) fn spawn(options: CheckerBuilder<M>) -> Self {
         let model = Arc::new(options.model);
+        let symmetry = options.symmetry;
         let target_state_count = options.target_state_count;
         let thread_count = options.thread_count;
         let visitor = Arc::new(options.visitor);
         let property_count = model.properties().len();
 
         let init_states: Vec<_> = model.init_states().into_iter()
-            .filter(|s| model.within_boundary(&s))
+            .filter(|s| model.within_boundary(s))
             .collect();
         let state_count = Arc::new(AtomicUsize::new(init_states.len()));
         let generated = Arc::new({
             let generated = DashSet::default();
-            for s in &init_states { generated.insert(fingerprint(s)); }
+            for s in &init_states {
+                if let Some(representative) = symmetry {
+                    generated.insert(fingerprint(&representative(s)));
+                } else {
+                    generated.insert(fingerprint(s));
+                }
+            }
             generated
         });
         let ebits = {
@@ -59,8 +66,8 @@ where M: Model + Send + Sync + 'static,
         };
         let pending: Vec<_> = init_states.into_iter()
             .map(|s| {
-                let fs = vec![fingerprint(&s)];
-                (s, fs, ebits.clone())
+                let fp = fingerprint(&s);
+                (s, vec![fp], ebits.clone())
             })
             .collect();
         let discoveries = Arc::new(DashMap::default());
@@ -116,7 +123,8 @@ where M: Model + Send + Sync + 'static,
                         &mut pending,
                         &*discoveries,
                         &*visitor,
-                        1500);
+                        1500,
+                        symmetry);
                     if discoveries.len() == property_count {
                         log::debug!("{}: Discovery complete. Shutting down... gen={}", t, generated.len());
                         let mut job_market = job_market.lock();
@@ -161,6 +169,8 @@ where M: Model + Send + Sync + 'static,
         }
     }
 
+    #[allow(clippy::too_many_arguments)]
+    #[allow(clippy::type_complexity)]
     fn check_block(
         model: &M,
         state_count: &AtomicUsize,
@@ -168,8 +178,9 @@ where M: Model + Send + Sync + 'static,
         pending: &mut Job<M::State>,
         discoveries: &DashMap<&'static str, Vec<Fingerprint>>,
         visitor: &Option<Box<dyn CheckerVisitor<M> + Send + Sync>>,
-        mut max_count: usize)
-    {
+        mut max_count: usize,
+        symmetry: Option<fn(&M::State) -> M::State>,
+    ) {
         let properties = model.properties();
 
         let mut actions = Vec::new();
@@ -222,15 +233,18 @@ where M: Model + Send + Sync + 'static,
                         }
                     }
                 }
-
             }
             if !is_awaiting_discoveries { return }
 
             // Otherwise enqueue newly generated states (with related metadata).
             let mut is_terminal = true;
             model.actions(&state, &mut actions);
-            let next_states = actions.drain(..).flat_map(|a| model.next_state(&state, a));
-            for next_state in next_states {
+            for action in actions.drain(..) {
+                let next_state = match model.next_state(&state, action) {
+                    None => continue,
+                    Some(next_state) => next_state,
+                };
+
                 // Skip if outside boundary.
                 if !model.within_boundary(&next_state) { continue }
                 state_count.fetch_add(1, Ordering::Relaxed);
@@ -243,19 +257,32 @@ where M: Model + Send + Sync + 'static,
                 // property held on the path leading to the first visit as meaning
                 // that it holds in the path leading to the second visit -- another
                 // possible false-negative.
-                let next_fingerprint = fingerprint(&next_state);
-                if !generated.insert(next_fingerprint) {
-                    // FIXME: arriving at an already-known state may be a loop (in which case it
-                    // could, in a fancier implementation, be considered a terminal state for
-                    // purposes of eventually-property checking) but it might also be a join in
-                    // a DAG, which makes it non-terminal. These cases can be disambiguated (at
-                    // some cost), but for now we just _don't_ treat them as terminal, and tell
-                    // users they need to explicitly ensure model path-acyclicality when they're
-                    // using eventually properties (using a boundary or empty actions or
-                    // whatever).
-                    is_terminal = false;
-                    continue
-                }
+                let next_fingerprint = if let Some(representative) = symmetry {
+                    let representative_fingerprint = fingerprint(&representative(&next_state));
+                    if !generated.insert(representative_fingerprint) {
+                        is_terminal = false;
+                        continue
+                    }
+                    // IMPORTANT: continue the path with the pre-canonicalized state/fingerprint to
+                    // avoid jumping to another part of the state space for which there may not be
+                    // a path extension from the previously collected path.
+                    fingerprint(&next_state)
+                } else {
+                    let next_fingerprint = fingerprint(&next_state);
+                    if !generated.insert(next_fingerprint) {
+                        // FIXME: arriving at an already-known state may be a loop (in which case it
+                        // could, in a fancier implementation, be considered a terminal state for
+                        // purposes of eventually-property checking) but it might also be a join in
+                        // a DAG, which makes it non-terminal. These cases can be disambiguated (at
+                        // some cost), but for now we just _don't_ treat them as terminal, and tell
+                        // users they need to explicitly ensure model path-acyclicality when they're
+                        // using eventually properties (using a boundary or empty actions or
+                        // whatever).
+                        is_terminal = false;
+                        continue
+                    }
+                    next_fingerprint
+                };
 
                 // Otherwise further checking is applicable.
                 is_terminal = false;
@@ -343,7 +370,7 @@ mod test {
         let checker = LinearEquation { a: 2, b: 4, c: 7 }.checker().spawn_dfs().join();
         assert_eq!(checker.is_done(), true);
         checker.assert_no_discovery("solvable");
-        assert_eq!(checker.generated_count(), 256 * 256);
+        assert_eq!(checker.unique_state_count(), 256 * 256);
     }
 
     #[test]
@@ -363,4 +390,95 @@ mod test {
             Guess::IncreaseX,
         ]);
     }
+
+    #[test]
+    fn can_apply_symmetry_reduction() {
+        use crate::actor::Id;
+        #[derive(Clone, Debug, Eq, Hash, PartialEq)]
+        struct Sys;
+        #[derive(Clone, Debug, Eq, Hash, PartialEq)]
+        struct SysState(Vec<ProcState>);
+        #[derive(Clone, Debug, Eq, Hash, Ord, PartialEq, PartialOrd)]
+        enum ProcState {
+            // A process advances from `Loading` to `Running` to cycling between `Paused` and
+            // `Running`. There is no way for a process to move from `Loading` to `Paused` or any
+            // non-`Loading` state to `Loading`, but a previous implementation of symmetry
+            // reduction would mistakenly collect a path with an invalid step because it would
+            // enqueue the representative of a state rather than the original state.
+            //
+            // Here is an example of the steps that would manifest that bug:
+            //
+            // 1. System starts: `[Loading, Loading]`
+            // 2. Second process advances: `[Loading, Running]`
+            // 3. Second process advances: `[Loading, Paused]`
+            //
+            // But in the third state above, the representative function swaps the process order
+            // because `Paused < Loading`, so the collected path becomes:
+            //
+            // 1. `[Loading, Loading]`
+            // 2. `[Loading, Running]`
+            // 3. `[Paused, Loading]`
+            //
+            // Both `Loading -> Loading -> Paused` (process 0) and `Loading -> Running -> Loading`
+            // (process 1) are invalid.
+            Paused,
+            Loading,
+            Running,
+        }
+        impl Model for Sys {
+            type Action = Id;
+            type State = SysState;
+            fn init_states(&self) -> Vec<SysState> {
+                vec![SysState(vec![ProcState::Loading, ProcState::Loading])]
+            }
+            fn actions(&self, _: &Self::State, actions: &mut Vec<Self::Action>) {
+                // Either process can run next.
+                actions.push(Id::from(0));
+                actions.push(Id::from(1));
+            }
+            fn next_state(&self, state: &Self::State, action: Self::Action) -> Option<Self::State> {
+                let i = usize::from(action);
+                let mut state = state.clone();
+                match state.0[i]  {
+                    ProcState::Loading => state.0[i] = ProcState::Running,
+                    ProcState::Running => state.0[i] = ProcState::Paused,
+                    ProcState::Paused => state.0[i] = ProcState::Running,
+                }
+                Some(state)
+            }
+            fn properties(&self) -> Vec<Property<Self>> {
+                vec![
+                    Property::<Self>::always(
+                        "visit all states",
+                        |_, _| true),
+                    Property::<Self>::sometimes(
+                        "a process pauses",
+                        |_, s| s.0[0] == ProcState::Paused || s.0[1] == ProcState::Paused),
+                ]
+            }
+        }
+        impl Representative for SysState {
+            fn representative(&self) -> Self {
+                let plan = RewritePlan::from_values_to_sort(&self.0);
+                SysState(plan.reindex(&self.0))
+            }
+        }
+        impl Rewrite<Id> for ProcState {
+            fn rewrite<S>(&self, _: &RewritePlan<Id, S>) -> Self {
+                self.clone()
+            }
+        }
+
+        // 9 states without symmetry reduction.
+        let checker = Sys.checker().spawn_dfs().join();
+        assert_eq!(checker.unique_state_count(), 9);
+        let checker = Sys.checker().spawn_bfs().join();
+        assert_eq!(checker.unique_state_count(), 9);
+
+        // 6 states with symmetry reduction. `PathRecorder` panics upon encountering an invalid
+        // path, and this was observed in a previous implementation with a bug.
+        let (visitor, _) = PathRecorder::new_with_accessor();
+        let checker = Sys.checker().symmetry().visitor(visitor).spawn_dfs().join();
+        assert_eq!(checker.unique_state_count(), 6);
+    }
 }
